/**
 * Voicer converts text into human-like speech in more than 200 voices across 30+ languages and variants.
 * Exclusively on Envato Market: https://1.envato.market/voicer
 *
 * @encoding        UTF-8
 * @version         2.0.2
 * @copyright       Copyright (C) 2018 - 2020 Merkulove (https://merkulov.design/). All rights reserved.
 * @license         Envato License https://1.envato.market/KYbje
 * @contributors    Alexander Khmelnitskiy (info@alexander.khmelnitskiy.ua), Dmitry Merkulov (dmitry@merkulov.design)
 * @support         help@merkulov.design
 **/

let AudioContext = window.AudioContext || window.webkitAudioContext || false;
let mdp_voicer_context = new AudioContext();

const VoicerSelection = (function () {

    "use strict";
    
    let mdp_voicer_active_source = null;

    /**
     * Detect touch device
     * @type {boolean}
     */
    let USER_IS_TOUCHING = false;

    function _selection() {
        const menu = {
            gspeak: true,
            disable: false
        };
        
        const gSpeakConfig = {
            icon: '<svg class="selection__icon" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" enable-background="new 0 0 24 24" width="24" height="24"><path d="M19.3,10.7L9.1,4.2C8.8,4,8.4,4,8.1,4C7,4,7,4.9,7,5.1v13.4c0,0.2,0,1.1,1.1,1.1c0.3,0,0.7,0,1-0.2l10.2-6.5c0.8-0.5,0.7-1.1,0.7-1.1S20.1,11.2,19.3,10.7z"/></svg>',
            preloader_icon: '<svg class="selection__icon" id="mdp-voicer-preloader" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="24" height="24"><rect x="3" y="7" rx="2" ry="2" width="4" height="10"><animate attributeName="y" calcMode="spline" values="7;9;7" keyTimes="0;0.5;1" dur=".6" keySplines="0 0.5 0.5 1;0 0.5 0.5 1" begin="0" repeatCount="indefinite"/><animate attributeName="height" calcMode="spline" values="10;6;10" keyTimes="0;0.5;1" dur=".6" keySplines="0 0.5 0.5 1;0 0.5 0.5 1" begin="0" repeatCount="indefinite"/></rect><rect x="8" y="2" rx="2" ry="2" width="4" height="20"><animate attributeName="y" calcMode="spline" values="2;4;2" keyTimes="0;0.5;1" dur=".5" keySplines="0 0.5 0.5 1;0 0.5 0.5 1" begin="0" repeatCount="indefinite"/><animate attributeName="height" calcMode="spline" values="20;16;20" keyTimes="0;0.5;1" dur=".5" keySplines="0 0.5 0.5 1;0 0.5 0.5 1" begin="0" repeatCount="indefinite"/></rect><rect x="13" y="4" rx="2" ry="2" width="4" height="16"><animate attributeName="y" calcMode="spline" values="4;7;4" keyTimes="0;0.5;1" dur=".7" keySplines="0 0.5 0.5 1;0 0.5 0.5 1" begin="0" repeatCount="indefinite"/><animate attributeName="height" calcMode="spline" values="16;10;16" keyTimes="0;0.5;1" dur=".7" keySplines="0 0.5 0.5 1;0 0.5 0.5 1" begin="0" repeatCount="indefinite"/></rect><rect x="18" y="8" rx="2" ry="2" width="4" height="8"><animate attributeName="y" calcMode="spline" values="8;10;8" keyTimes="0;0.5;1" dur=".8" keySplines="0 0.5 0.5 1;0 0.5 0.5 1" begin="0" repeatCount="indefinite"/><animate attributeName="height" calcMode="spline" values="8;4;8" keyTimes="0;0.5;1" dur=".8" keySplines="0 0.5 0.5 1;0 0.5 0.5 1" begin="0" repeatCount="indefinite"/></rect></svg>',
            stop_icon: '<svg class="selection__icon" xmlns="http://www.w3.org/2000/svg" width="24" height="24"><path d="M18.001 4.2H6A1.8 1.8 0 0 0 4.2 5.999V18A1.8 1.8 0 0 0 5.999 19.8H18a1.8 1.8 0 0 0 1.799-1.799V6c0-.992-.807-1.799-1.799-1.799z"/></svg>'
        };
        
        let selection = '';
        let selectionHTML = '';
        let text = '';
        let bgcolor = mdp_voicer.bgcolor;
        let iconcolor = mdp_voicer.iconcolor;

        let _icons = {};
        let arrowsize = 5;
        let buttonmargin = 7 * 2;
        let iconsize = 24 + buttonmargin;
        let top = 0;
        let left = 0;
                
        function gspeakButton() {
            const gspbtn = new Button( gSpeakConfig.icon, function ( ) {
                
                /** Stop playing. */
                if ( document.querySelector( '.mdp-voicer-selection.stop' ) ) {
                    stopActiveSource();
                    onVoiceEnded();
                    return;
                }
                
                /** Set 'loading' icon. */
                setPreloaderIcon();

                /** AJAX Request with vanila JS. */
                var request = new XMLHttpRequest();

                request.open( 'POST', mdp_voicer.ajaxurl, true );
                request.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded;');
                request.onload = function () {

                    /** If successful. */
                    if ( this.status >= 200 && this.status < 400 ) {
                        
                        if ( ! AudioContext ) {
                            console.log( 'Error with creating AudioContext.' );
                            return;
                        }
                        
                        mdp_voicer_context.decodeAudioData( this.response, function( buffer ) {
                            const source = mdp_voicer_context.createBufferSource();
                            source.buffer = buffer;
                              stopActiveSource(); // Stop the active one if any.
                              mdp_voicer_active_source = source;
                              source.connect( mdp_voicer_context.destination );
                              source.onended = onVoiceEnded;
                              source.start( 0 );
                              
                              /** Set 'stop' icon. */
                              setStopIcon();
                              
                        }, function( e ){ 
                            console.log( 'Error with decoding audio data' + e.err );
                        });
                        
                    } else {
                        /** If fail. */
                        console.log( 'Fail:' );
                        console.log( this.response );
                    }
                };
                request.onerror = function() {
                    /** Connection error. */
                    console.log( 'Connection error.' );                    
                };
                request.responseType = 'arraybuffer';
                request.send( 'action=voicergspeak&text=' + encodeURIComponent( text ) );
                                
            } );
            return gspbtn;
        }
        
        /** On Playback Finished. */
        function onVoiceEnded() {
            
            /** Hide tooltip, if new selection wasn't made. */
            if ( document.querySelector( '.mdp-voicer-selection.stop'  ) ) {
                document.querySelector('.mdp-voicer-selection').remove();
            }
            
        }
        
        /** Set Play Icon . */
        function setPlayIcon() {
            /** Select old icon that will be replaced. */
            const el = document.querySelector('.mdp-voicer-selection .selection__icon');

            /** Create a new icon that will take the place of "el". */
            const newEl = document.createElement('div');
            newEl.innerHTML = gSpeakConfig.icon;

            /** Replace el with newEL. */
            el.parentNode.replaceChild( newEl, el );

            /** Mark tooltip as 'Play' button. */
            const stop_popup = document.querySelector('.mdp-voicer-selection');
            stop_popup.classList.remove( 'stop' );
            stop_popup.classList.remove( 'loading' );
            
        }
        
        /** Set Preloader Icon . */
        function setPreloaderIcon() {

            /** Select old icon that will be replaced. */
            const el = document.querySelector('.mdp-voicer-selection .selection__icon');

            /** Create a new icon that will take the place of "el". */
            const newEl = document.createElement('div');
            newEl.innerHTML = gSpeakConfig.preloader_icon;

            /** Replace el with newEL. */
            el.parentNode.replaceChild( newEl, el );

            /** Mark tooltip as 'Stop' button. */
            const stop_popup = document.querySelector('.mdp-voicer-selection');
            stop_popup.classList.remove( 'stop' );
            stop_popup.classList.add( 'loading' );
            
        }

        /** Set Stop Icon. */
        function setStopIcon() {

            /** Select old icon that will be replaced. */
            const el = document.querySelector('.mdp-voicer-selection .selection__icon');

            /** Create a new icon that will take the place of "el". */
            const newEl = document.createElement('div');
            newEl.innerHTML = gSpeakConfig.stop_icon;

            /** Replace el with newEL. */
            el.parentNode.replaceChild(newEl, el);

            /** Mark tooltip as 'Stop' button. */
            const stop_popup = document.querySelector('.mdp-voicer-selection');
            stop_popup.classList.remove( 'loading' );
            stop_popup.classList.add( 'stop' );
            
        }

        function IconStyle() {
            const style = document.createElement('style');
            style.innerHTML = `.selection__icon{fill:${iconcolor};}`;
            document.body.appendChild(style);
        }

        function appendIcons() {
            const myitems = [
                { feature: 'gspeak', call: gspeakButton() }
            ];
            const div = document.createElement('div');
            let count = 0;
            myitems.forEach((item) => {
                if (menu[item.feature]) {
                    div.appendChild(item.call);
                    count++;
                }
            });
            return {
                icons: div,
                length: count
            };
        }

        function setTooltipPosition() {
            const position = selection.getRangeAt(0).getBoundingClientRect();
            const DOCUMENT_SCROLL_TOP = window.pageXOffset || document.documentElement.scrollTop || document.body.scrollTop;

            left = position.left + (position.width - iconsize * _icons.length) / 2;

            /** Set position for desktop **/
            if ( ! USER_IS_TOUCHING ) {

                top = position.top + DOCUMENT_SCROLL_TOP - iconsize - arrowsize;

            /** Set position for mobile **/
            } else {

                top = position.bottom + DOCUMENT_SCROLL_TOP + iconsize + arrowsize;

            }

        }

        function moveTooltip() {
            
            if ( !!document.querySelector('.mdp-voicer-selection') ) { 
                setTooltipPosition();
                let tooltip = document.querySelector('.mdp-voicer-selection');
                tooltip.style.top = `${top}px`;
                tooltip.style.left = `${left}px`;
            }
            
        }

        function drawTooltip() {
            _icons = appendIcons();
            setTooltipPosition();

            /** Set transition for mobile **/
            let mobileTransform = '';
            if ( USER_IS_TOUCHING ) {
                mobileTransform = 'transform-origin: top; transform: scale(1,-1)';
            }

            const div = document.createElement('div');
            div.className = 'mdp-voicer-selection';
            div.style =
                'line-height: 0;' +
                'position:absolute;' +
                'background-color:' + bgcolor + ';' +
                'border-radius:20px;' +
                'top:' + top + 'px;' +
                'left:' + left + 'px;' +
                'transition:all .2s ease-in-out;' +
                'box-shadow: 0 0px 24px rgba(0,0,0,0.25);' +
                'z-index:99999;' +
                mobileTransform;

            div.appendChild( _icons.icons );

            const arrow = document.createElement( 'div' );
            arrow.style =
                'position:absolute;' +
                'border-left:' + arrowsize + 'px solid transparent;' +
                'border-right:' + arrowsize +'px solid transparent;' +
                'border-top:' + arrowsize + 'px solid ' + bgcolor + ';' +
                'bottom:-' + ( arrowsize - 1 ) + 'px;' +
                'left:' + ( iconsize * _icons.length / 2 - arrowsize ) + 'px;' +
                'width:0;' +
                'height:0;';

            if ( ! menu.disable ) {
                div.appendChild( arrow );
            }

            document.body.appendChild( div );
        }

        function attachEvents() {
            function hasSelection() {
                return !!window.getSelection().toString();
            }

            function hasTooltipDrawn() {
                return !!document.querySelector('.mdp-voicer-selection');
            }

            /** Return HTML of selected text. */
            function getHTMLOfSelection () {
                let range;
                if (document.selection && document.selection.createRange) {
                    range = document.selection.createRange();
                    return range.htmlText;
                }
                else if (window.getSelection) {
                    const selection = window.getSelection();
                    if (selection.rangeCount > 0) {
                        range = selection.getRangeAt(0);
                        const clonedSelection = range.cloneContents();
                        const div = document.createElement('div');
                        div.appendChild(clonedSelection);
                        return div.innerHTML;
                    }
                    else {
                        return '';
                    }
                }
                else {
                    return '';
                }
            }
            
            /** Selecting the full word if only its parts are selected. */
            function snapSelectionToWord() {
                let sel;

                /** Do not Modify Selection for Touch devices. */
                if ( USER_IS_TOUCHING ) {
                    if( window.getSelection ) {
                        sel = window.getSelection();
                    } else if ( document.getSelection ) {
                        sel = document.getSelection();
                    } else if ( document.selection ) {
                        sel = document.selection.createRange().text;
                    }
                    return sel;
                }

                /** 
                 * Check for existence of window.getSelection() and that it has a
                 * modify() method. IE 9 has both selection APIs but no modify() method.
                 **/
                if ( window.getSelection && (sel = window.getSelection()).modify ) {
                    sel = window.getSelection();
                    if ( ! sel.isCollapsed ) {

                        /** Detect if selection is backwards. */
                        const range = document.createRange();
                        range.setStart(sel.anchorNode, sel.anchorOffset);
                        range.setEnd(sel.focusNode, sel.focusOffset);
                        const backwards = range.collapsed;
                        range.detach();

                        /** Modify() works on the focus of the selection. */
                        const endNode = sel.focusNode, endOffset = sel.focusOffset;
                        sel.collapse(sel.anchorNode, sel.anchorOffset);

                        let direction = [];
                        if (backwards) {
                            direction = ['backward', 'forward'];
                        } else {
                            direction = ['forward', 'backward'];
                        }

                        sel.modify( 'move', direction[0], 'character' );
                        sel.modify( 'move', direction[1], 'word' );
                        sel.extend( endNode, endOffset);
                        sel.modify( 'extend', direction[1], 'character' );
                        sel.modify( 'extend', direction[0], 'word' );
                    }
                } else if ( ( sel = document.selection ) && sel.type !== 'Control' ) {
                    const textRange = sel.createRange();
                    if ( textRange.text ) {
                        textRange.expand( 'word' );
                        
                        /** Move the end back to not include the word's trailing space(s), if necessary. */
                        while ( /\s$/.test( textRange.text ) ) {
                            textRange.moveEnd( 'character', -1 );
                        }
                        textRange.select();
                    }
                }
                
                return sel;
            }

            const onMouseUp = function () {

                setTimeout(function mouseTimeout() {
                    if (hasTooltipDrawn()) {

                        if (hasSelection()) {
                            selection = snapSelectionToWord();

                            selectionHTML = getHTMLOfSelection();
                            text = selectionHTML;

                            moveTooltip();

                            /** If now playing, set icon to play. */
                            if (
                                (document.querySelector('.mdp-voicer-selection.stop')) ||
                                (document.querySelector('.mdp-voicer-selection.loading'))
                            ) {

                                /** Set ion Play. */
                                setPlayIcon();
                            }

                        } else {

                            /** Hide tooltip,  If we now not paying. */
                            if (
                                (!document.querySelector('.mdp-voicer-selection.stop')) &&
                                (!document.querySelector('.mdp-voicer-selection.loading'))
                            ) {
                                document.querySelector('.mdp-voicer-selection').remove();
                            }

                        }

                    } else if (hasSelection()) {
                        selection = snapSelectionToWord();
                        selectionHTML = getHTMLOfSelection();
                        text = selectionHTML;
                        drawTooltip();
                    }
                }, 10);

            };

            window.addEventListener( 'mouseup', onMouseUp, true );
            window.addEventListener( 'touchend', onMouseUp, false );
            window.addEventListener( 'touchcancel', onMouseUp, false );
            window.addEventListener( 'selectionchange', onMouseUp, false );
            
            window.addEventListener( 'resize', moveTooltip, false );
            
            /** We want to detect human touch, not device touch. */
            window.addEventListener( 'touchstart', function onFirstTouch() {
                
                /** Set global flag. */
                USER_IS_TOUCHING = true;
                
                document.addEventListener( 'selectionchange', onMouseUp, true );
                
                /** We only need to know once that a human touched the screen, so we can stop listening now. */
                window.removeEventListener( 'touchstart', onFirstTouch, false );
            } );
            
        }

        function config(options) {
            menu.gspeak = options.gspeak === undefined ? menu.gspeak : options.gspeak;
            menu.disable = options.disable === undefined ? menu.disable : options.disable;

            bgcolor = options.backgroundColor || '#333';
            iconcolor = options.iconColor || '#fff';
            return this;
        }

        function init() {
            IconStyle();
            attachEvents();
            return this;
        }

        return {
            config: config,
            init: init
        };
    }

    /**
     * Render button.
     * @param icon - SVG icon
     * @param clickFn - Click event handler
     * @return {HTMLDivElement}
     * @constructor
     */
    function Button( icon, clickFn ) {

        const btn = document.createElement('div');
        btn.style.display = 'display:inline-block';
        btn.style.margin = '7px';
        btn.style.cursor = 'pointer';
        btn.style.transition = 'all .2s ease-in-out';
        btn.innerHTML = icon;
        btn.onclick = clickFn;

        if ( btn.id === 'mdp-voicer-preloader' ) {
            btn.style.transition = 'none';
        }

        else {
            btn.onmouseover = function () {
                this.style.transform = 'scale(1.2)';
            };
            btn.onmouseout = function () {
                this.style.transform = 'scale(1)';
            };
        }

        return btn;
    }

    /**
     * Stop the active one if any.
     */
    function stopActiveSource() {
        if (mdp_voicer_active_source) {
            mdp_voicer_active_source.onended = null; // manual stop, no event
            mdp_voicer_active_source.stop(0);
        }
    }

    return _selection;
    
})();

/** Unlocking Web Audio for f#cking Safari. */
const webAudioTouchUnlock = function() {

    "use strict";

    mdp_voicer_context.resume();

};

/** Run once the page DOM is ready for JavaScript code to execute. */
document.addEventListener( 'DOMContentLoaded', function() {

    "use strict";

    const mdp_voicer_selection = new VoicerSelection();
    mdp_voicer_selection.init();
    
    document.body.addEventListener( 'click', webAudioTouchUnlock, false );

} );
